"use strict";

exports.__esModule = true;
exports["default"] = void 0;

var _ramda = require("ramda");

var _ramdaAdjunct = require("ramda-adjunct");

// Type definitions:
//     Resource = Object
//     ResourceRel = {type: String, id: Number}
//     ResourceKey = String
//     Relationships = Object
//     Included = [Resource]
//     IncludedCache = {k: ResourceKey, v: Resource}
//     RelPath = {path: String, key: ResourceKey}
// getRelationships :: Resource -> Relationships
var getRelationships = (0, _ramda.pipe)((0, _ramda.propOr)({}, ['relationships']), _ramda.keys); // resourceKey :: ResourceRel -> ResourceKey

var resourceKey = function resourceKey(_ref) {
  var type = _ref.type,
      id = _ref.id;
  return "".concat(type, "-").concat(id);
}; // createIncludedCache :: Included -> IncludedCache


var createIncludedCache = (0, _ramda.reduce)(function (acc, resource) {
  return (0, _ramda.assoc)(resourceKey(resource), resource, acc);
}, {}); // relToPath :: (String, ResourceRel) -> RelPath

var relToPath = function relToPath(relationship, _ref2) {
  var type = _ref2.type,
      id = _ref2.id;
  return {
    path: ['relationships', relationship, 'data'],
    key: resourceKey({
      type: type,
      id: id
    })
  };
}; // relToPaths :: (String, [ResourceRel]) -> [RelPath]


var relToPaths = function relToPaths(relationship, resources) {
  return (0, _ramdaAdjunct.mapIndexed)(function (resource, index) {
    return {
      path: ['relationships', relationship, 'data', index],
      key: resourceKey(resource)
    };
  }, resources);
}; // getRelPaths :: Resource -> [RelPath]


var getRelPaths = function getRelPaths(resource) {
  var relationships = getRelationships(resource);
  return (0, _ramda.reduce)(function (acc, relationship) {
    var relData = resource.relationships[relationship].data;
    var paths = (0, _ramdaAdjunct.ensureArray)((0, _ramdaAdjunct.isArray)(relData) ? relToPaths(relationship, relData) : relToPath(relationship, relData));
    return (0, _ramda.concat)(acc, paths);
  }, [], relationships);
}; // jsonApiMergeSingle :: (Included, Resource) -> Resource


var jsonApiMergeSingle = function jsonApiMergeSingle(includedCache, resource) {
  var relPaths = getRelPaths(resource);
  return (0, _ramda.reduce)(function (acc, _ref3) {
    var key = _ref3.key,
        path = _ref3.path;
    var includedResource = (0, _ramda.has)(key, includedCache) ? includedCache[key] : (0, _ramda.path)(path, resource);
    return (0, _ramda.assocPath)(path, includedResource, acc);
  }, resource, relPaths);
}; // jsonApiMergeMulti :: (IncludedCache, [Resource]) -> [Resource]


var jsonApiMergeMulti = function jsonApiMergeMulti(includedCache, resources) {
  return (0, _ramdaAdjunct.reduceIndexed)(function (acc, datum, index) {
    var mergedDatum = jsonApiMergeSingle(includedCache, datum);
    return (0, _ramda.insert)(index, mergedDatum, acc);
  }, [], resources);
}; // jsonApiMerge :: (Included, Resource) -> Resource
// jsonApiMerge :: (Included, [Resource]) -> [Resource]


var jsonApiMerge = (0, _ramda.curry)(function (included, resource) {
  if ((0, _ramda.isNil)(included)) return resource;
  var includedCache = createIncludedCache(included);
  return (0, _ramdaAdjunct.isArray)(resource) ? jsonApiMergeMulti(includedCache, resource) : jsonApiMergeSingle(includedCache, resource);
});
var _default = jsonApiMerge;
exports["default"] = _default;